# Common makefile for Extensible Interpreter Design Kit using Watcom compilers

CUR_DIR =     $+ $(%cwd) $-
XIDEK = $(CUR_DIR)\..\..\..

SUPPORT_DIR = $(XIDEK)\support\$(SUPPORT_EXT)
LANG_DIR =    $(XIDEK)\$(LANG)\$(LANG_EXT)
DRIVER_DIR =  $(XIDEK)\driver
AGCLIB_DIR =  $(XIDEK)\agclib1
TEST_DIR =    $(XIDEK)\test\$(LANG)

!ifndef RES
RES = r3
!endif

.ERASE
.BEFORE
	set INCLUDE=$(SUPPORT_DIR);$(LANG_DIR);$(AGCLIB_DIR)\include;$(%INCLUDE)

CL = wcl386
CC = wpp386

CFLAGS = -xs -bt=nt -5r -d2

OBJS = demo.obj comdefs.obj bcidefs.obj astdefs.obj ast.obj&
       dxi.obj dci.obj astxi.obj astci.obj

SUPPORT_OBJS     = comdefs.obj bcidefs.obj astdefs.obj astxi.obj astci.obj
LANG_OBJS        = ast.obj dxi.obj dci.obj
DRIVER_OBJS      = demo.obj

DXI_OBJS   = demo.obj dxi.obj   comdefs.obj
DCI_OBJS   = demo.obj dci.obj   comdefs.obj bcidefs.obj
ASTXI_OBJS = demo.obj astxi.obj comdefs.obj ast.obj astdefs.obj
ASTCI_OBJS = demo.obj astci.obj comdefs.obj ast.obj astdefs.obj bcidefs.obj

AGCLIB = $(AGCLIB_DIR)\watcom\agclib1.lib

PROGS  = dxi.exe dci.exe astxi.exe astci.exe

DX = t1.dx t2.dx t3.dx
DC = t1.dc t2.dc t3.dc
AX = t1.ax t2.ax t3.ax
AC = t1.ac t2.ac t3.ac

TESTS = $(DX) $(DC) $(AX) $(AC) $(EXT_TESTS)

# the first rule is the one you get when you type "make" by itself
all: progs tests .SYMBOLIC

progs: $(PROGS) .SYMBOLIC

tests: $(TESTS) .SYMBOLIC

retest: clearTests $(TESTS) .SYMBOLIC

# Link phase. $< is the full right-hand side of the rule.
dxi.exe: $(DXI_OBJS) $(AGCLIB) .AUTODEPEND
	$(CL) $(CFLAGS) -fe=$@ $<

dci.exe: $(DCI_OBJS) $(AGCLIB) .AUTODEPEND
	$(CL) $(CFLAGS) -fe=$@ $<

astxi.exe: $(ASTXI_OBJS) $(AGCLIB) .AUTODEPEND
	$(CL) $(CFLAGS) -fe=$@ $<

astci.exe: $(ASTCI_OBJS) $(AGCLIB) .AUTODEPEND
	$(CL) $(CFLAGS) -fe=$@ $<

$(AGCLIB):
	cd $(AGCLIB_DIR)\watcom
	wmake
	cd $(CUR_DIR)

# Compile phase

#Locate cpp files
.cpp: $(SUPPORT_DIR);$(LANG_DIR);$(DRIVER_DIR)

#Rules for compiling
.cpp.obj :  .AUTODEPEND
	set command=$(CFLAGS) $[@
	$(CC) @command

#Parser Build Phase
.EXTENSIONS: .syn

#Explicit dependencies, needed to make parser builds work
dxi.obj : $(LANG_DIR)\dxi.cpp $(LANG_DIR)\dxi.h .AUTODEPEND
dci.obj : $(LANG_DIR)\dci.cpp $(LANG_DIR)\dci.h .AUTODEPEND
ast.obj : $(LANG_DIR)\ast.cpp $(LANG_DIR)\ast.h .AUTODEPEND

#Locate syntax files
.syn: $(LANG_DIR);

#Rules for building parsers
.syn.cpp:
	cd $(LANG_DIR)
	agcl $^&
	cd $(CUR_DIR)

#Tests
.EXTENSIONS: .dx .dc .ax .ac .scf

#Dependencies
t1.dx: $(TEST_DIR)\t1.scf  dxi.exe
t1.dc: $(TEST_DIR)\t1.scf  dci.exe
t1.ax: $(TEST_DIR)\t1.scf  astxi.exe
t1.ac: $(TEST_DIR)\t1.scf  astci.exe

t2.dx: $(TEST_DIR)\t2.scf  dxi.exe
t2.dc: $(TEST_DIR)\t2.scf  dci.exe
t2.ax: $(TEST_DIR)\t2.scf  astxi.exe
t2.ac: $(TEST_DIR)\t2.scf  astci.exe

t3.dx: $(TEST_DIR)\t3.scf  dxi.exe
t3.dc: $(TEST_DIR)\t3.scf  dci.exe
t3.ax: $(TEST_DIR)\t3.scf  astxi.exe
t3.ac: $(TEST_DIR)\t3.scf  astci.exe

#Locate test files
.scf: $(TEST_DIR)

#Rules for running tests
.scf.dx:
	dxi $[@ > $^&.dx
	fc $^&.dx $(TEST_DIR)\$^&.$(RES)

.scf.dc:
	dci $[@ > $^&.dc
	fc $^&.dc $(TEST_DIR)\$^&.$(RES)

.scf.ax:
	astxi $[@ > $^&.ax
	fc $^&.ax $(TEST_DIR)\$^&.$(RES)

.scf.ac:
	astci $[@ > $^&.ac
	fc $^&.ac $(TEST_DIR)\$^&.$(RES)

# Remove object files and library file for a fresh build
# clean is the standard target name for "remove objects".
clean: clearTests .SYMBOLIC
	del $(OBJS)
        del $(PROGS)

clearTests : .SYMBOLIC
	del *.dx *.dc *.ax *.ac
